﻿// Copyright (c) 2004-2008 MySQL AB, 2008-2009 Sun Microsystems, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data.Common;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Text;
using MySql.Data.MySqlClient.Properties;

namespace MySql.Data.MySqlClient
{
    class MySqlConnectionStringBuilder : DbConnectionStringBuilder
    {
        private static Dictionary<string, PropertyInfo> validKeywords =
            new Dictionary<string, PropertyInfo>(StringComparer.OrdinalIgnoreCase);
        private static Dictionary<string, object> defaultValues =
            new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);
        private Dictionary<string, object> values =
            new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);
         
        static MySqlConnectionStringBuilder()
        {
            // load up our valid keywords and default values only once
            Initialize();
        }

        public MySqlConnectionStringBuilder()
        {
            Clear();
        }

        public MySqlConnectionStringBuilder(string connStr)
            : this()
        {
            ConnectionString = connStr;
        }

        #region Server Properties

        /// <summary>
        /// Gets or sets the name of the server.
        /// </summary>
        /// <value>The server.</value>
        [Category("Connection")]
        [Description("Server to connect to")]
        [DefaultValue("")]
        [ValidKeywords("host, data source, datasource, address, addr, network address")]
        [RefreshProperties(RefreshProperties.All)]
        public string Server
        {
            get { return values["server"] as string; }
            set { SetValue("server", value); }
        }

        /// <summary>
        /// Gets or sets the name of the database the connection should 
        /// initially connect to.
        /// </summary>
        [Category("Connection")]
        [Description("Database to use initially")]
        [DefaultValue("")]
        [ValidKeywords("initial catalog")]
        [RefreshProperties(RefreshProperties.All)]
        public string Database
        {
            get { return values["database"] as string; }
            set { SetValue("database", value); }
        }

        /// <summary>
        /// Gets or sets the protocol that should be used for communicating
        /// with MySQL.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Connection Protocol")]
        [Description("Protocol to use for connection to MySQL")]
        [DefaultValue(MySqlConnectionProtocol.Sockets)]
        [ValidKeywords("protocol, connection protocol")]
        [RefreshProperties(RefreshProperties.All)]
        public MySqlConnectionProtocol ConnectionProtocol
        {
            get { return (MySqlConnectionProtocol)values["ConnectionProtocol"]; }
            set { SetValue("ConnectionProtocol", value); }
        }

        /// <summary>
        /// Gets or sets the name of the named pipe that should be used
        /// for communicating with MySQL.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Pipe Name")]
        [Description("Name of pipe to use when connecting with named pipes (Win32 only)")]
        [DefaultValue("MYSQL")]
        [ValidKeywords("pipe Name, pipe")]
        [RefreshProperties(RefreshProperties.All)]
        public string PipeName
        {
            get { return (string)values["pipeName"]; }
            set { SetValue("PipeName", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether this connection
        /// should use compression.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Use Compression")]
        [Description("Should the connection ues compression")]
        [DefaultValue(false)]
        [ValidKeywords("compress, use compression")]
        [RefreshProperties(RefreshProperties.All)]
        public bool UseCompression
        {
            get { return (bool)values["UseCompression"]; }
            set { SetValue("UseCompression", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether this connection will allow
        /// commands to send multiple SQL statements in one execution.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Allow Batch")]
        [Description("Allows execution of multiple SQL commands in a single statement")]
        [DefaultValue(true)]
        [ValidKeywords("allow batch")]
        [RefreshProperties(RefreshProperties.All)]
        public bool AllowBatch
        {
            get { return (bool)values["AllowBatch"]; }
            set { SetValue("AllowBatch", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether logging is enabled.
        /// </summary>
        [Category("Connection")]
        [Description("Enables output of diagnostic messages")]
        [DefaultValue(false)]
        [RefreshProperties(RefreshProperties.All)]
        public bool Logging
        {
            get { return (bool)values["Logging"]; }
            set { SetValue("Logging", value); }
        }

        /// <summary>
        /// Gets or sets the base name of the shared memory objects used to 
        /// communicate with MySQL when the shared memory protocol is being used.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Shared Memory Name")]
        [Description("Name of the shared memory object to use")]
        [DefaultValue("MYSQL")]
        [ValidKeywords("shared memory name")]
        [RefreshProperties(RefreshProperties.All)]
        public string SharedMemoryName
        {
            get { return (string)values["SharedMemoryName"]; }
            set { SetValue("SharedMemoryName", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates whether this connection uses
        /// the old style (@) parameter markers or the new (?) style.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Use Old Syntax")]
        [Description("Allows the use of old style @ syntax for parameters")]
        [DefaultValue(false)]
        [ValidKeywords("use old syntax, old syntax, oldsyntax")]
        [RefreshProperties(RefreshProperties.All)]
        [Obsolete("Use Old Syntax is no longer needed.  See documentation")]
        public bool UseOldSyntax
        {
            get { return (bool)values["UseOldSyntax"]; }
            set { SetValue("UseOldSyntax", value); }
        }

        /// <summary>
        /// Gets or sets the port number that is used when the socket
        /// protocol is being used.
        /// </summary>
        [Category("Connection")]
        [Description("Port to use for TCP/IP connections")]
        [DefaultValue(3306)]
        [RefreshProperties(RefreshProperties.All)]
        public uint Port
        {
            get { return (uint)values["Port"]; }
            set { SetValue("Port", value); }
        }

        /// <summary>
        /// Gets or sets the connection timeout.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Connect Timeout")]
        [Description("The length of time (in seconds) to wait for a connection " +
                     "to the server before terminating the attempt and generating an error.")]
        [DefaultValue(15)]
        [ValidKeywords("connection timeout, connect timeout")]
        [RefreshProperties(RefreshProperties.All)]
        public uint ConnectionTimeout
        {
            get { return (uint)values["ConnectionTimeout"]; }
            set { SetValue("ConnectionTimeout", value); }
        }

        /// <summary>
        /// Gets or sets the default command timeout.
        /// </summary>
        [Category("Connection")]
        [DisplayName("Default Command Timeout")]
        [Description(@"The default timeout that MySqlCommand objects will use
                     unless changed.")]
        [DefaultValue(30)]
        [ValidKeywords("default command timeout, command timeout")]
        [RefreshProperties(RefreshProperties.All)]
        public uint DefaultCommandTimeout
        {
            get { return (uint)values["DefaultCommandTimeout"]; }
            set { SetValue("DefaultCommandTimeout", value); }
        }

        #endregion

        #region Authentication Properties

        /// <summary>
        /// Gets or sets the user id that should be used to connect with.
        /// </summary>
        [Category("Security")]
        [DisplayName("User Id")]
        [Description("Indicates the user ID to be used when connecting to the data source.")]
        [DefaultValue("")]
        [ValidKeywords("uid, username, user name, user id, user")]
        [RefreshProperties(RefreshProperties.All)]
        public string UserID
        {
            get { return (string)values["UserId"]; }
            set { SetValue("UserId", value); }
        }

        /// <summary>
        /// Gets or sets the password that should be used to connect with.
        /// </summary>
        [Category("Security")]
        [Description("Indicates the password to be used when connecting to the data source.")]
        [PasswordPropertyText(true)]
        [DefaultValue("")]
        [ValidKeywords("pwd")]
        [RefreshProperties(RefreshProperties.All)]
        public string Password
        {
            get { return (string)values["Password"]; }
            set { SetValue("Password", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value that indicates if the password should be persisted
        /// in the connection string.
        /// </summary>
        [Category("Security")]
        [DisplayName("Persist Security Info")]
        [Description("When false, security-sensitive information, such as the password, " +
                     "is not returned as part of the connection if the connection is open or " +
                     "has ever been in an open state.")]
        [DefaultValue(false)]
        [ValidKeywords("persist security info")]
        [RefreshProperties(RefreshProperties.All)]
        public bool PersistSecurityInfo
        {
            get { return (bool)values["PersistSecurityInfo"]; }
            set { SetValue("PersistSecurityInfo", value); }
        }

        [Category("Authentication")]
        [Description("Should the connection use SSL.")]
        [DefaultValue(false)]
        [Obsolete("Use Ssl Mode instead.")]
        internal bool Encrypt
        {
            get { return SslMode != MySqlSslMode.None; }
            set
            {
                SetValue("SslMode", value ? MySqlSslMode.Prefered : MySqlSslMode.None);
            }
        }

        #endregion

        #region Other Properties

        /// <summary>
        /// Gets or sets a boolean value that indicates if zero date time values are supported.
        /// </summary>
        [Category("Advanced")]
        [DisplayName("Allow Zero Datetime")]
        [Description("Should zero datetimes be supported")]
        [DefaultValue(false)]
        [ValidKeywords("allow zero datetime")]
        [RefreshProperties(RefreshProperties.All)]
        public bool AllowZeroDateTime
        {
            get { return (bool)values["AllowZeroDatetime"]; }
            set { SetValue("AllowZeroDateTime", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating if zero datetime values should be 
        /// converted to DateTime.MinValue.
        /// </summary>
        [Category("Advanced")]
        [DisplayName("Convert Zero Datetime")]
        [Description("Should illegal datetime values be converted to DateTime.MinValue")]
        [DefaultValue(false)]
        [ValidKeywords("convert zero datetime")]
        [RefreshProperties(RefreshProperties.All)]
        public bool ConvertZeroDateTime
        {
            get { return (bool)values["ConvertZeroDatetime"]; }
            set { SetValue("ConvertZeroDateTime", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating if the Usage Advisor should be enabled.
        /// </summary>
        [Category("Advanced")]
        [DisplayName("Use Usage Advisor")]
        [Description("Logs inefficient database operations")]
        [DefaultValue(false)]
        [ValidKeywords("usage advisor, use usage advisor")]
        [RefreshProperties(RefreshProperties.All)]
        public bool UseUsageAdvisor
        {
            get { return (bool)values["UseUsageAdvisor"]; }
            set { SetValue("UseUsageAdvisor", value); }
        }

        /// <summary>
        /// Gets or sets the size of the stored procedure cache.
        /// </summary>
        [Category("Advanced")]
        [DisplayName("Procedure Cache Size")]
        [Description("Indicates how many stored procedures can be cached at one time. " +
                     "A value of 0 effectively disables the procedure cache.")]
        [DefaultValue(25)]
        [ValidKeywords("procedure cache size, procedure cache, procedurecache")]
        [RefreshProperties(RefreshProperties.All)]
        public uint ProcedureCacheSize
        {
            get { return (uint)values["ProcedureCacheSize"]; }
            set { SetValue("ProcedureCacheSize", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating if the permon hooks should be enabled.
        /// </summary>
        [Category("Advanced")]
        [DisplayName("Use Performance Monitor")]
        [Description("Indicates that performance counters should be updated during execution.")]
        [DefaultValue(false)]
        [ValidKeywords("use performance monitor, userperfmon, perfmon")]
        [RefreshProperties(RefreshProperties.All)]
        public bool UsePerformanceMonitor
        {
            get { return (bool)values["UsePerformanceMonitor"]; }
            set { SetValue("UsePerformanceMonitor", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating if calls to Prepare() should be ignored.
        /// </summary>
        [Category("Advanced")]
        [DisplayName("Ignore Prepare")]
        [Description("Instructs the provider to ignore any attempts to prepare a command.")]
        [DefaultValue(true)]
        [ValidKeywords("ignore prepare")]
        [RefreshProperties(RefreshProperties.All)]
        public bool IgnorePrepare
        {
            get { return (bool)values["IgnorePrepare"]; }
            set { SetValue("IgnorePrepare", value); }
        }

        [Category("Advanced")]
        [DisplayName("Use Procedure Bodies")]
        [Description("Indicates if stored procedure bodies will be available for parameter detection.")]
        [DefaultValue(true)]
        [ValidKeywords("use procedure bodies, procedure bodies")]
        [RefreshProperties(RefreshProperties.All)]
        public bool UseProcedureBodies
        {
            get { return (bool)values["UseProcedureBodies"]; }
            set { SetValue("UseProcedureBodies", value); }
        }

        [Category("Advanced")]
        [DisplayName("Auto Enlist")]
        [Description("Should the connetion automatically enlist in the active connection, if there are any.")]
        [DefaultValue(true)]
        [ValidKeywords("auto enlist")]
        [RefreshProperties(RefreshProperties.All)]
        public bool AutoEnlist
        {
            get { return (bool)values["AutoEnlist"]; }
            set { SetValue("AutoEnlist", value); }
        }

        [Category("Advanced")]
        [DisplayName("Respect Binary Flags")]
        [Description("Should binary flags on column metadata be respected.")]
        [DefaultValue(true)]
        [ValidKeywords("respect binary flags")]
        [RefreshProperties(RefreshProperties.All)]
        public bool RespectBinaryFlags
        {
            get { return (bool)values["RespectBinaryFlags"]; }
            set { SetValue("RespectBinaryFlags", value); }
        }

        [Category("Advanced")]
        [DisplayName("Treat Tiny As Boolean")]
        [Description("Should the provider treat TINYINT(1) columns as boolean.")]
        [DefaultValue(true)]
        [ValidKeywords("treat tiny as boolean")]
        [RefreshProperties(RefreshProperties.All)]
        public bool TreatTinyAsBoolean
        {
            get { return (bool)values["TreatTinyAsBoolean"]; }
            set { SetValue("TreatTinyAsBoolean", value); }
        }

        [Category("Advanced")]
        [DisplayName("Allow User Variables")]
        [Description("Should the provider expect user variables to appear in the SQL.")]
        [DefaultValue(false)]
        [ValidKeywords("allow user variables")]
        [RefreshProperties(RefreshProperties.All)]
        public bool AllowUserVariables
        {
            get { return (bool)values["AllowUserVariables"]; }
            set { SetValue("AllowUserVariables", value); }
        }

        [Category("Advanced")]
        [DisplayName("Interactive Session")]
        [Description("Should this session be considered interactive?")]
        [DefaultValue(false)]
        [ValidKeywords("interactive, interactive session")]
        [RefreshProperties(RefreshProperties.All)]
        public bool InteractiveSession
        {
            get { return (bool)values["InteractiveSession"]; }
            set { SetValue("InteractiveSession", value); }
        }

        [Category("Advanced")]
        [DisplayName("Functions Return String")]
        [Description("Should all server functions be treated as returning string?")]
        [DefaultValue(false)]
        [ValidKeywords("functions return string")]
        public bool FunctionsReturnString
        {
            get { return (bool)values["FunctionsReturnString"]; }
            set { SetValue("FunctionsReturnString", value); }
        }

        [Category("Advanced")]
        [DisplayName("Use Affected Rows")]
        [Description("Should the returned affected row count reflect affected rows instead of found rows?")]
        [DefaultValue(false)]
        [ValidKeywords("use affected rows")]
        public bool UseAffectedRows
        {
            get { return (bool)values["UseAffectedRows"]; }
            set { SetValue("UseAffectedRows", value); }
        }


        [Category("Advanced")]
        [DisplayName("Old Guids")]
        [Description("Treat binary(16) columns as guids")]
        [DefaultValue(false)]
        [ValidKeywords("old guids")]
        public bool OldGuids
        {
            get { return (bool)values["OldGuids"]; }
            set { SetValue("OldGuids", value); }
        }

        [DisplayName("Keepalive")]
        [Description("For TCP connections, idle connection time measured in seconds, before the first keepalive packet is sent." +
            "A value of 0 indicates that keepalive is not used.")]
        [DefaultValue(0)]
        [ValidKeywords("Keep Alive")]
        public uint Keepalive
        {
            get { return (uint)values["KeepAlive"]; }
            set { SetValue("KeepAlive", value); }
        }

        #endregion

        #region Pooling Properties

        /// <summary>
        /// Gets or sets the lifetime of a pooled connection.
        /// </summary>
        [Category("Pooling")]
        [DisplayName("Connection Lifetime")]
        [Description("The minimum amount of time (in seconds) for this connection to " +
                     "live in the pool before being destroyed.")]
        [DefaultValue(0)]
        [ValidKeywords("connection lifetime")]
        [RefreshProperties(RefreshProperties.All)]
        public uint ConnectionLifeTime
        {
            get { return (uint)values["ConnectionLifeTime"]; }
            set { SetValue("ConnectionLifeTime", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating if connection pooling is enabled.
        /// </summary>
        [Category("Pooling")]
        [Description("When true, the connection object is drawn from the appropriate " +
                     "pool, or if necessary, is created and added to the appropriate pool.")]
        [DefaultValue(true)]
        [RefreshProperties(RefreshProperties.All)]
        public bool Pooling
        {
            get { return (bool)values["Pooling"]; }
            set { SetValue("Pooling", value); }
        }

        /// <summary>
        /// Gets the minimum connection pool size.
        /// </summary>
        [Category("Pooling")]
        [DisplayName("Minimum Pool Size")]
        [Description("The minimum number of connections allowed in the pool.")]
        [DefaultValue(0)]
        [ValidKeywords("min pool size, minimum pool size")]
        [RefreshProperties(RefreshProperties.All)]
        public uint MinimumPoolSize
        {
            get { return (uint)values["MinimumPoolSize"]; }
            set { SetValue("MinimumPoolSize", value); }
        }

        /// <summary>
        /// Gets or sets the maximum connection pool setting.
        /// </summary>
        [Category("Pooling")]
        [DisplayName("Maximum Pool Size")]
        [Description("The maximum number of connections allowed in the pool.")]
        [DefaultValue(100)]
        [ValidKeywords("max pool size, maxium pool size")]
        [RefreshProperties(RefreshProperties.All)]
        public uint MaximumPoolSize
        {
            get { return (uint)values["MaximumPoolSize"]; }
            set { SetValue("MaximumPoolSize", value); }
        }

        /// <summary>
        /// Gets or sets a boolean value indicating if the connection should be reset when retrieved
        /// from the pool.
        /// </summary>
        [Category("Pooling")]
        [DisplayName("Connection Reset")]
        [Description("When true, indicates the connection state is reset when " +
                     "removed from the pool.")]
        [DefaultValue(true)]
        [ValidKeywords("connection reset")]
        [RefreshProperties(RefreshProperties.All)]
        public bool ConnectionReset
        {
            get { return (bool)values["ConnectionReset"]; }
            set { SetValue("ConnectionReset", value); }
        }

        #endregion

        #region Language and Character Set Properties

        /// <summary>
        /// Gets or sets the character set that should be used for sending queries to the server.
        /// </summary>
        [DisplayName("Character Set")]
        [Category("Advanced")]
        [Description("Character set this connection should use")]
        [DefaultValue("")]
        [ValidKeywords("character set, charset")]
        [RefreshProperties(RefreshProperties.All)]
        public string CharacterSet
        {
            get { return (string)values["CharacterSet"]; }
            set { SetValue("CharacterSet", value); }
        }

        /// <summary>
        /// Indicates whether the driver should treat binary blobs as UTF8
        /// </summary>
        [DisplayName("Treat Blobs As UTF8")]
        [Category("Advanced")]
        [Description("Should binary blobs be treated as UTF8")]
        [DefaultValue(false)]
        [ValidKeywords("treat blobs as utf8")]
        [RefreshProperties(RefreshProperties.All)]
        public bool TreatBlobsAsUTF8
        {
            get { return (bool)values["TreatBlobsAsUTF8"]; }
            set { SetValue("TreatBlobsAsUTF8", value); }
        }

        /// <summary>
        /// Gets or sets the pattern that matches the columns that should be treated as UTF8
        /// </summary>
        [DisplayName("BlobAsUTF8IncludePattern")]
        [Category("Advanced")]
        [Description("Pattern that matches columns that should be treated as UTF8")]
        [DefaultValue("")]
        [RefreshProperties(RefreshProperties.All)]
        public string BlobAsUTF8IncludePattern
        {
            get { return (string)values["BlobAsUTF8IncludePattern"]; }
            set { SetValue("BlobAsUTF8IncludePattern", value); }
        }

        /// <summary>
        /// Gets or sets the pattern that matches the columns that should not be treated as UTF8
        /// </summary>
        [DisplayName("BlobAsUTF8ExcludePattern")]
        [Category("Advanced")]
        [Description("Pattern that matches columns that should not be treated as UTF8")]
        [DefaultValue("")]
        [RefreshProperties(RefreshProperties.All)]
        public string BlobAsUTF8ExcludePattern
        {
            get { return (string)values["BlobAsUTF8ExcludePattern"]; }
            set { SetValue("BlobAsUTF8ExcludePattern", value); }
        }

        /// <summary>
        /// Indicates whether to use SSL connections and how to handle server certificate errors.
        /// </summary>
        [DisplayName("SslMode")]
        [Category("Security")]
        [Description("SSL properties for connection")]
        [DefaultValue(MySqlSslMode.None)]
        [ValidKeywords("ssl mode")]
        public MySqlSslMode SslMode
        {
            get { return (MySqlSslMode)values["SslMode"]; }
            set { SetValue("SslMode", value); }
        }

        #endregion

        internal Regex GetBlobAsUTF8IncludeRegex()
        {
            if (BlobAsUTF8IncludePattern == null) return null;
            return new Regex(BlobAsUTF8IncludePattern);
        }

        internal Regex GetBlobAsUTF8ExcludeRegex()
        {
            if (BlobAsUTF8ExcludePattern == null) return null;
            return new Regex(BlobAsUTF8ExcludePattern);
        }

        public override object this[string keyword]
        {
            get { return base[keyword]; }
            set
            {
                ValidateKeyword(keyword);
                if (value == null)
                    Remove(keyword);
                else
                    SetValue(keyword, value);
            }
        }

        public override void Clear()
        {
            base.Clear();

            // make a copy of our default values array
            foreach (string key in defaultValues.Keys)
                values[key] = defaultValues[key];
        }

        public override bool Remove(string keyword)
        {
            ValidateKeyword(keyword);
            if (!base.Remove(keyword)) return false;
            values.Remove(keyword);
            values[keyword] = defaultValues[validKeywords[keyword].Name];
            return true;
        }

        public string GetConnectionString(bool includePass)
        {
            if (includePass) return ConnectionString;

            StringBuilder conn = new StringBuilder();
            string delimiter = "";
            foreach (string key in this.Keys)
            {
                if (String.Compare(key, "password", true) == 0 ||
                    String.Compare(key, "pwd", true) == 0) continue;
                conn.AppendFormat("{0}{1}={2}", delimiter, key, this[key]);
                delimiter = ";";
            }
            return conn.ToString();
        }

        private void SetValue(string keyword, object value)
        {
            ValidateKeyword(keyword);
            object val = Convert.ChangeType(value, validKeywords[keyword].PropertyType);
            values[keyword] = val;
            base[keyword] = val;
        }

        private void ValidateKeyword(string keyword)
        {
            string key = keyword.ToLowerInvariant();
            if (!validKeywords.ContainsKey(key))
                throw new ArgumentException(Resources.KeywordNotSupported, keyword);
            if (validKeywords[key].Name == "UseOldSyntax")
                Logger.LogWarning("Use Old Syntax is now obsolete.  Please see documentation");
            if (validKeywords[key].Name == "Encrypt")
                Logger.LogWarning("Encrypt is now obsolete. Use Ssl Mode instead");
        }

        private static void Initialize()
        {
            PropertyInfo[] properties = typeof(MySqlConnectionStringBuilder).GetProperties();
            foreach (PropertyInfo pi in properties)
            {
                string name = pi.Name.ToLowerInvariant();
                validKeywords.Add(name, pi);

                foreach (Attribute a in pi.GetCustomAttributes(false))
                {
                    if (a is ValidKeywordsAttribute)
                    {
                        foreach (string keyword in (a as ValidKeywordsAttribute).Keywords)
                            validKeywords.Add(keyword, pi);
                    }
                    else if (a is DefaultValueAttribute)
                        defaultValues[name] = Convert.ChangeType((a as DefaultValueAttribute).Value,
                            pi.PropertyType);
                }
            }
        }
    }

    internal class ValidKeywordsAttribute : Attribute
    {
        private string keywords;

        public ValidKeywordsAttribute(string keywords)
        {
            this.keywords = keywords.ToLowerInvariant();
        }

        public string[] Keywords
        {
            get { return keywords.Split(','); }
        }
    }
}
